package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.sort.SortEnum;
import de.uni_marburg.powersort.data.ObjectSupplier;

import java.util.concurrent.TimeUnit;

/**
 * Custom Benchmark (CBM).
 */
public class CbmBase {
    static void run(SortEnum[] sortImplementations, DataEnum[] dataEnums) {
        System.out.println();
        for (DataEnum dataEnum : dataEnums) {
            ObjectSupplier objectSupplier = null;
            for (SortEnum sortImplementation : sortImplementations) {
                if (!Filter.isFiltered(dataEnum, sortImplementation)) {
                    if (objectSupplier == null) {
                        System.out.println("⏳ Creating object supplier " + dataEnum + " ⏳");
                        objectSupplier = dataEnum.getObjectSupplier();
                    }
                    benchmark(objectSupplier, sortImplementation);
                }
            }

            System.out.println();
        }
    }

    static void benchmark(ObjectSupplier objectSupplier, SortEnum sortImplementation) {
        System.out.print(sortImplementation);
        System.out.flush();
        Object[] sortInput = objectSupplier.getCopy();

        // TODO: JVM warmup!
        final long startNanos = System.nanoTime();
        sortImplementation.getSortImpl().sort(sortInput);
        final long stopNanos = System.nanoTime();

        final long durNanos = stopNanos - startNanos;
        final long durMillis = TimeUnit.NANOSECONDS.toMillis(durNanos);
        final String durFormatted = LongFormatter.formatUnderscore(durMillis);

        System.out.println("," + durFormatted);
    }
}
